import json
from shlex import shlex
from datetime import datetime
from typing import Union, Any, Dict

from pytz import utc

from kakaowork.consts import KST, BOOL_STRS, TRUE_STRS


def is_bool(text: Union[str, bytes]) -> bool:
    s = text.decode('utf-8') if isinstance(text, bytes) else text
    return s.strip().lower() in BOOL_STRS


def is_int(text: Union[str, bytes]) -> bool:
    s = text.decode('utf-8') if isinstance(text, bytes) else text
    try:
        int(s)
    except ValueError:
        return False
    return True


def is_float(text: Union[str, bytes]) -> bool:
    s = text.decode('utf-8') if isinstance(text, bytes) else text
    try:
        return str(float(s)) == s
    except ValueError:
        return False


def text2bool(text: Union[str, bytes]) -> bool:
    s = text.decode('utf-8') if isinstance(text, bytes) else text
    return s.strip().lower() in TRUE_STRS


def text2dict(text: Union[str, bytes]) -> Dict[str, Any]:
    json_str = text.decode('utf-8') if isinstance(text, bytes) else text
    return json.loads(json_str)


def exist_kv(key: str, node: Dict[str, Any]) -> bool:
    if key in node and node[key]:
        return True
    return False


def to_kst(timestamp: Union[int, datetime]) -> datetime:
    if isinstance(timestamp, int):
        return datetime.fromtimestamp(timestamp, tz=utc).astimezone(KST)
    elif isinstance(timestamp, datetime):
        # If the timestamp is naive then just replace tzinfo to KST
        # ref: https://docs.python.org/3/library/datetime.html#determining-if-an-object-is-aware-or-naive
        if timestamp.tzinfo is None or timestamp.tzinfo.utcoffset(timestamp) is None:
            return KST.localize(timestamp)
        else:
            return timestamp.astimezone(KST)
    raise ValueError('Unsupported timestamp type')


def normalize_token(token: str) -> str:
    token = token.lower()
    if '_' in token:
        return token.replace('_', '-')
    return token


def command_aliases() -> Dict[str, str]:
    from kakaowork.command import users, conversations, messages, departments, spaces, bots
    aliases: Dict[str, str] = {}
    aliases.update({alias: users.name for alias in ('user', )})
    aliases.update({alias: conversations.name
                    for alias in (
                        'conversation',
                        'conv',
                    )})
    aliases.update({alias: messages.name
                    for alias in (
                        'message',
                        'msg',
                    )})
    aliases.update({alias: departments.name
                    for alias in (
                        'department',
                        'dept',
                    )})
    aliases.update({alias: spaces.name for alias in ('space', )})
    aliases.update({alias: bots.name for alias in ('bot', )})
    return aliases


def parse_kv_pairs(line: str) -> Dict[str, Any]:
    lexer = shlex(line, posix=True)
    lexer.wordchars += "=.-_()/:+*^&%$#@!?|{}"
    kvs: Dict[str, Any] = {}
    for token in lexer:
        key, value = token.split('=', maxsplit=1)
        if is_bool(value):
            kvs[key] = text2bool(value)
        elif is_int(value):
            kvs[key] = int(value)
        elif is_float(value):
            kvs[key] = float(value)
        else:
            kvs[key] = value
    return kvs


def json_default(value: Any) -> Any:
    from kakaowork.blockkit import Block

    if isinstance(value, Block):
        return value.to_dict()
    elif isinstance(value, datetime):
        return int(value.timestamp())
    raise TypeError('not JSON serializable')



from enum import Enum
from datetime import timedelta

from pytz import timezone


class StrEnum(str, Enum):
    """
    String enumeration type
    """
    pass


LIMIT = 10

BASE_URL = 'https://api.kakaowork.com'
BASE_PATH_USERS = '/v1/users'
BASE_PATH_CONVERSATIONS = '/v1/conversations'
BASE_PATH_MESSAGES = '/v1/messages'
BASE_PATH_DEPARTMENTS = '/v1/departments'
BASE_PATH_SPACES = '/v1/spaces'
BASE_PATH_BOTS = '/v1/bots'

KST = timezone('Asia/Seoul')

TRUE_STRS = ['true', 'y', 'yes']
FALSE_STRS = ['false', 'n', 'no']
BOOL_STRS = TRUE_STRS + FALSE_STRS



import json
from abc import ABC, abstractclassmethod
from datetime import datetime
from typing import Optional, NamedTuple, Union, Any, Dict, List

from pytz import utc, timezone

from kakaowork.consts import StrEnum
from kakaowork.blockkit import Block, BlockType
from kakaowork.exceptions import NoValueError
from kakaowork.utils import text2dict, exist_kv, to_kst, json_default


class ErrorCode(StrEnum):
    INVALID_PARAMETER = 'invalid_parameter'
    INVALID_AUTHENTICATION = 'invalid_authentication'
    INVALID_REPRESENTATION = 'invalid_representation'
    INVALID_CONTENT_TYPE = 'invalid_content_type'
    API_NOT_FOUND = 'api_not_found'
    UNAUTHORIZED = 'unauthorized'
    INTERNAL_SERVER_ERROR = 'internal_server_error'
    TOO_MANY_REQUESTS = 'too_many_requests'
    EXPIRED_AUTHENTICATION = 'expired_authentication'
    MISSING_PARAMETER = 'missing_parameter'
    CONVERSATION_NOT_FOUND = 'conversation_not_found'


class ConversationType(StrEnum):
    DM = 'dm'
    GROUP = 'group'


class ColorTone(StrEnum):
    LIGHT = 'light'
    DARK = 'dark'


class ProfileNameFormat(StrEnum):
    NAME_ONLY = 'name_only'
    NAME_NICKNAME = 'name_nickname'
    NICKNAME_NAME = 'nickname_name'


class ProfilePositionFormat(StrEnum):
    POSITION = 'position'
    RESPONSIBILITY = 'responsibility'


class BotStatus(StrEnum):
    ACTIVATED = 'activated'
    DEACTIVATED = 'deactivated'


class ErrorField(NamedTuple):
    code: ErrorCode
    message: str

    def to_dict(self) -> Dict[str, Any]:
        return self._asdict()

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'ErrorField':
        if not value:
            raise NoValueError('No value to type cast')
        return cls(**dict(
            value,
            code=ErrorCode(value['code']) if exist_kv('code', value) else None,
        ))


class UserIdentificationField(NamedTuple):
    type: str
    value: str

    def to_dict(self) -> Dict[str, Any]:
        return self._asdict()

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'UserIdentificationField':
        if not value:
            raise NoValueError('No value to type cast')
        return cls(**dict(value))


class UserField(NamedTuple):
    id: str
    space_id: str
    name: str
    identifications: Optional[List[UserIdentificationField]] = None
    nickname: Optional[str] = None
    avatar_url: Optional[str] = None
    department: Optional[str] = None
    position: Optional[str] = None
    responsibility: Optional[str] = None
    tels: Optional[str] = None
    mobiles: Optional[str] = None
    work_start_time: Optional[datetime] = None
    work_end_time: Optional[datetime] = None
    vacation_start_time: Optional[datetime] = None
    vacation_end_time: Optional[datetime] = None

    def to_dict(self) -> Dict[str, Any]:
        return dict(
            self._asdict(),
            identifications=[uid.to_dict() for uid in self.identifications] if self.identifications else None,
        )

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'UserField':
        if not value:
            raise NoValueError('No value to type cast')
        return cls(**dict(
            value,
            identifications=[UserIdentificationField.from_dict(item) for item in value['identifications']] if exist_kv('identifications', value) else None,
            work_start_time=to_kst(value['work_start_time']) if exist_kv('work_start_time', value) else None,
            work_end_time=to_kst(value['work_end_time']) if exist_kv('work_end_time', value) else None,
            vacation_start_time=to_kst(value['vacation_start_time']) if exist_kv('vacation_start_time', value) else None,
            vacation_end_time=to_kst(value['vacation_end_time']) if exist_kv('vacation_end_time', value) else None,
        ))


class ConversationField(NamedTuple):
    id: str
    type: ConversationType
    users_count: int
    avatar_url: Optional[str] = None
    name: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        return self._asdict()

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'ConversationField':
        if not value:
            raise NoValueError('No value to type cast')
        return cls(**dict(
            value,
            type=ConversationType(value['type']),
        ))


class MessageField(NamedTuple):
    id: str
    text: str
    user_id: str
    conversation_id: int
    send_time: datetime
    update_time: datetime
    blocks: Optional[List[Block]] = None

    def to_dict(self) -> Dict[str, Any]:
        return dict(
            self._asdict(),
            blocks=[b.to_dict() for b in self.blocks] if self.blocks else None,
        )

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'MessageField':
        if not value:
            raise NoValueError('No value to type cast')

        blocks: List[Block] = []
        if exist_kv('blocks', value):
            for kv in value['blocks']:
                block_cls = BlockType.block_cls(kv['type'])
                blocks.append(block_cls.from_dict(kv))
        return cls(**dict(
            value,
            send_time=to_kst(value['send_time']) if exist_kv('send_time', value) else None,
            update_time=to_kst(value['update_time']) if exist_kv('update_time', value) else None,
            blocks=blocks,
        ))


class DepartmentField(NamedTuple):
    id: str
    ids_path: str
    parent_id: str
    space_id: str
    name: str
    code: str
    user_count: int
    has_child: Optional[bool] = None
    depth: Optional[int] = None
    users_ids: Optional[List[int]] = None
    leader_ids: Optional[List[int]] = None
    ancestry: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        return self._asdict()

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'DepartmentField':
        if not value:
            raise NoValueError('No value to type cast')
        return cls(**dict(value))


class SpaceField(NamedTuple):
    id: int
    kakaoi_org_id: int
    name: str
    color_code: str
    color_tone: ColorTone
    permitted_ext: List[str]
    profile_name_format: ProfileNameFormat
    profile_position_format: ProfilePositionFormat
    logo_url: str

    def to_dict(self) -> Dict[str, Any]:
        return self._asdict()

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'SpaceField':
        if not value:
            raise NoValueError('No value to type cast')
        return cls(**dict(
            value,
            color_tone=ColorTone(value['color_tone']),
            profile_name_format=ProfileNameFormat(value['profile_name_format']),
            profile_position_format=ProfilePositionFormat(value['profile_position_format']),
        ))


class BotField(NamedTuple):
    bot_id: int
    title: str
    status: BotStatus

    def to_dict(self) -> Dict[str, Any]:
        return self._asdict()

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'BotField':
        if not value:
            raise NoValueError('No value to type cast')
        return cls(**dict(
            value,
            status=BotStatus(value['status']),
        ))


class BaseResponse(ABC, object):
    def __init__(self, *, success: Optional[bool] = None, error: Optional[ErrorField] = None):
        self.success = success if success is not None else True
        self.error = error

    def __str__(self):
        return self.to_json()

    def __repr__(self):
        return str(self)

    def __eq__(self, value: object) -> bool:
        if not isinstance(value, BaseResponse):
            return False
        return self.to_dict() == value.to_dict()

    def to_json(self) -> str:
        return json.dumps(self.to_dict(), default=json_default)

    def to_dict(self) -> Dict[str, Any]:
        return {
            'success': self.success,
            'error': self.error.to_dict() if self.error else None,
        }

    def to_plain(self) -> str:
        if self.error:
            return '\n'.join([
                f'Error code:\t{self.error.code}',
                f'Message:\t{self.error.message}',
            ])
        return 'OK'

    @classmethod
    def from_json(cls, value: Union[str, bytes]) -> 'BaseResponse':
        data = text2dict(value)
        return cls(**dict(
            data,
            error=ErrorField.from_dict(data['error']) if exist_kv('error', data) else None,
        ))


class UserResponse(BaseResponse):
    def __init__(self, *, success: Optional[bool] = None, error: Optional[ErrorField] = None, user: Optional[UserField] = None):
        super().__init__(success=success, error=error)
        self.user = user

    def to_dict(self) -> Dict[str, Any]:
        return dict(
            super().to_dict(),
            user=self.user.to_dict() if self.user else None,
        )

    def to_plain(self) -> str:
        if self.user:
            return '\n'.join([
                f'ID:\t{self.user.id}',
                f'Name:\t{self.user.name}',
                f'Nickname:\t{self.user.nickname}',
                f'Department:\t{self.user.department}',
                f'Position:\t{self.user.position}',
                f'Responsibility:\t{self.user.responsibility}',
                f'Tels:\t{self.user.tels}',
                f'Mobiles:\t{self.user.mobiles}',
                f'Work start time:\t{self.user.work_start_time}',
                f'Work end time:\t{self.user.work_end_time}',
                f'Vacation start time:\t{self.user.vacation_start_time}',
                f'Vacation end time:\t{self.user.vacation_end_time}',
            ])
        return super().to_plain()

    @classmethod
    def from_json(cls, value: Union[str, bytes]) -> 'UserResponse':
        data = text2dict(value)
        r = cls(**dict(
            data,
            error=ErrorField.from_dict(data['error']) if exist_kv('error', data) else None,
            user=UserField.from_dict(data['user']) if exist_kv('user', data) else None,
        ))
        return r


class UserListResponse(BaseResponse):
    def __init__(self,
                 *,
                 success: Optional[bool] = None,
                 error: Optional[ErrorField] = None,
                 cursor: Optional[str] = None,
                 users: Optional[List[UserField]] = None):
        super().__init__(success=success, error=error)
        self.cursor = cursor
        self.users = users

    def to_dict(self) -> Dict[str, Any]:
        return dict(
            super().to_dict(),
            cursor=self.cursor,
            users=[u.to_dict() for u in self.users] if self.users else None,
        )

    def to_plain(self) -> str:
        if self.users:
            outs = []
            for user in self.users:
                out = '\n'.join([
                    f'ID:\t{user.id}',
                    f'Name:\t{user.name}',
                    f'Nickname:\t{user.nickname}',
                    f'Department:\t{user.department}',
                    f'Position:\t{user.position}',
                    f'Responsibility:\t{user.responsibility}',
                    f'Tels:\t{user.tels}',
                    f'Mobiles:\t{user.mobiles}',
                    f'Work start time:\t{user.work_start_time}',
                    f'Work end time:\t{user.work_end_time}',
                    f'Vacation start time:\t{user.vacation_start_time}',
                    f'Vacation end time:\t{user.vacation_end_time}',
                ])
                outs.append(out)
            return ('\n' + '-' * 30 + '\n').join(outs)
        return super().to_plain()

    @classmethod
    def from_json(cls, value: Union[str, bytes]) -> 'UserListResponse':
        data = text2dict(value)
        return cls(**dict(
            data,
            error=ErrorField.from_dict(data['error']) if exist_kv('error', data) else None,
            users=[UserField.from_dict(node) for node in data['users']] if exist_kv('users', data) else None,
        ))


class ConversationResponse(BaseResponse):
    def __init__(self, *, success: Optional[bool] = None, error: Optional[ErrorField] = None, conversation: Optional[ConversationField] = None):
        super().__init__(success=success, error=error)
        self.conversation = conversation

    def to_dict(self) -> Dict[str, Any]:
        return dict(
            super().to_dict(),
            conversation=self.conversation.to_dict() if self.conversation else None,
        )

    def to_plain(self) -> str:
        if self.conversation:
            return '\n'.join([
                f'ID:\t{self.conversation.id}',
                f'Type:\t{self.conversation.type}',
                f'Name:\t{self.conversation.name}',
                f'Avatar URL:\t{self.conversation.avatar_url}',
            ])
        return super().to_plain()

    @classmethod
    def from_json(cls, value: Union[str, bytes]) -> 'ConversationResponse':
        data = text2dict(value)
        return cls(**dict(
            data,
            error=ErrorField.from_dict(data['error']) if exist_kv('error', data) else None,
            conversation=ConversationField.from_dict(data['conversation']) if exist_kv('conversation', data) else None,
        ))


class ConversationListResponse(BaseResponse):
    def __init__(self,
                 *,
                 success: Optional[bool] = None,
                 error: Optional[ErrorField] = None,
                 cursor: Optional[str] = None,
                 conversations: Optional[List[ConversationField]] = None):
        super().__init__(success=success, error=error)
        self.cursor = cursor
        self.conversations = conversations

    def to_dict(self) -> Dict[str, Any]:
        return dict(
            super().to_dict(),
            cursor=self.cursor,
            conversations=[c.to_dict() for c in self.conversations] if self.conversations else None,
        )

    def to_plain(self) -> str:
        if self.conversations:
            outs = []
            for conversation in self.conversations:
                out = '\n'.join([
                    f'ID:\t{conversation.id}',
                    f'Type:\t{conversation.type}',
                    f'Name:\t{conversation.name}',
                    f'Avatar URL:\t{conversation.avatar_url}',
                ])
                outs.append(out)
            return ('\n' + '-' * 30 + '\n').join(outs)
        return super().to_plain()

    @classmethod
    def from_json(cls, value: Union[str, bytes]) -> 'ConversationListResponse':
        data = text2dict(value)
        return cls(**dict(
            data,
            error=ErrorField.from_dict(data['error']) if exist_kv('error', data) else None,
            conversations=[ConversationField.from_dict(node) for node in data['conversations']] if exist_kv('conversations', data) else None,
        ))


class MessageResponse(BaseResponse):
    def __init__(self, *, success: Optional[bool] = None, error: Optional[ErrorField] = None, message: Optional[MessageField] = None):
        super().__init__(success=success, error=error)
        self.message = message

    def to_dict(self):
        return dict(
            super().to_dict(),
            message=self.message.to_dict() if self.message else None,
        )

    def to_plain(self):
        if self.message:
            return '\n'.join([
                f'ID:\t{self.message.id}',
                f'Conversation ID:\t{self.message.conversation_id}',
                f'Send time:\t{self.message.send_time}',
                f'Update time:\t{self.message.update_time}',
                f'Text:\t{self.message.text}',
                f'Blocks:\t{len(self.message.blocks) if self.message.blocks else "-"}',
            ])
        return super().to_plain()

    @classmethod
    def from_json(cls, value: Union[str, bytes]) -> 'MessageResponse':
        data = text2dict(value)
        return cls(**dict(
            data,
            error=ErrorField.from_dict(data['error']) if exist_kv('error', data) else None,
            message=MessageField.from_dict(data['message']) if exist_kv('message', data) else None,
        ))


class DepartmentListResponse(BaseResponse):
    def __init__(self,
                 *,
                 success: Optional[bool] = None,
                 error: Optional[ErrorField] = None,
                 cursor: Optional[str] = None,
                 departments: Optional[List[DepartmentField]] = None):
        super().__init__(success=success, error=error)
        self.cursor = cursor
        self.departments = departments

    def to_dict(self) -> Dict[str, Any]:
        return dict(
            super().to_dict(),
            cursor=self.cursor,
            departments=[d.to_dict() for d in self.departments] if self.departments else None,
        )

    def to_plain(self) -> str:
        if self.departments:
            outs = []
            for department in self.departments:
                out = '\n'.join([
                    f'ID:\t\t{department.id}',
                    f'Name:\t\t{department.name}',
                    f'Code:\t\t{department.code}',
                    f'User count:\t{department.user_count}',
                ])
                outs.append(out)
            return ('\n' + '-' * 30 + '\n').join(outs)
        return super().to_plain()

    @classmethod
    def from_json(cls, value: Union[str, bytes]):
        data = text2dict(value)
        return cls(**dict(
            data,
            error=ErrorField.from_dict(data['error']) if exist_kv('error', data) else None,
            departments=[DepartmentField.from_dict(node) for node in data['departments']] if exist_kv('departments', data) else None,
        ))


class SpaceResponse(BaseResponse):
    def __init__(self, *, success: Optional[bool] = None, error: Optional[ErrorField] = None, space: Optional[SpaceField] = None):
        super().__init__(success=success, error=error)
        self.space = space

    def to_dict(self) -> Dict[str, Any]:
        return dict(
            super().to_dict(),
            space=self.space.to_dict() if self.space else None,
        )

    def to_plain(self) -> str:
        if self.space:
            return '\n'.join([
                f'ID:\t{self.space.id}',
                f'OrgID:\t{self.space.kakaoi_org_id}',
                f'Name:\t{self.space.name}',
                f'Color code:\t{self.space.color_code}',
                f'Color tone:\t{self.space.color_tone}',
                f'Permitted ext:\t{self.space.permitted_ext}',
                f'Profile name format:\t{self.space.profile_name_format}',
                f'Profile position format:\t{self.space.profile_position_format}',
                f'Logo URL:\t{self.space.logo_url}',
            ])
        return super().to_plain()

    @classmethod
    def from_json(cls, value: Union[str, bytes]) -> 'SpaceResponse':
        data = text2dict(value)
        return cls(**dict(
            data,
            error=ErrorField.from_dict(data['error']) if exist_kv('error', data) else None,
            space=SpaceField.from_dict(data['space']) if exist_kv('space', data) else None,
        ))


class BotResponse(BaseResponse):
    def __init__(self, *, success: Optional[bool] = None, error: Optional[ErrorField] = None, info: Optional[BotField] = None):
        super().__init__(success=success, error=error)
        self.info = info

    def to_dict(self) -> Dict[str, Any]:
        return dict(
            super().to_dict(),
            info=self.info.to_dict() if self.info else None,
        )

    def to_plain(self) -> str:
        if self.info:
            return '\n'.join([f'ID:\t{self.info.bot_id}', f'Name:\t{self.info.title}', f'Status:\t{self.info.status}'])
        return super().to_plain()

    @classmethod
    def from_json(cls, value: Union[str, bytes]) -> 'BotResponse':
        data = text2dict(value)
        return cls(**dict(
            data,
            error=ErrorField.from_dict(data['error']) if exist_kv('error', data) else None,
            info=BotField.from_dict(data['info']) if exist_kv('info', data) else None,
        ))



import os
import json
from enum import unique
from abc import ABC, abstractmethod, abstractclassmethod
from typing import Any, Dict, List, Optional, NamedTuple, Type, Union
from urllib.parse import urlparse

from kakaowork.consts import StrEnum
from kakaowork.exceptions import InvalidBlock, InvalidBlockType, NoValueError
from kakaowork.utils import exist_kv, json_default


@unique
class BlockType(StrEnum):
    TEXT = "text"
    IMAGE_LINK = "image_link"
    BUTTON = "button"
    DIVIDER = "divider"
    HEADER = "header"
    ACTION = "action"
    DESCRIPTION = "description"
    SECTION = "section"
    CONTEXT = "context"
    LABEL = "label"
    INPUT = "input"
    SELECT = "select"

    @classmethod
    def block_cls(cls, type: Union[str, 'BlockType']) -> Type['Block']:
        bt = cls(type) if isinstance(type, str) else type
        if bt == cls.TEXT:
            return TextBlock
        elif bt == cls.IMAGE_LINK:
            return ImageLinkBlock
        elif bt == cls.BUTTON:
            return ButtonBlock
        elif bt == cls.DIVIDER:
            return DividerBlock
        elif bt == cls.HEADER:
            return HeaderBlock
        elif bt == cls.ACTION:
            return ActionBlock
        elif bt == cls.DESCRIPTION:
            return DescriptionBlock
        elif bt == cls.SECTION:
            return SectionBlock
        elif bt == cls.CONTEXT:
            return ContextBlock
        elif bt == cls.LABEL:
            return LabelBlock
        elif bt == cls.INPUT:
            return InputBlock
        elif bt == cls.SELECT:
            return SelectBlock
        raise InvalidBlockType()


@unique
class ButtonStyle(StrEnum):
    DEFAULT = "default"
    PRIMARY = "primary"
    DANGER = "danger"


@unique
class ButtonActionType(StrEnum):
    OPEN_INAPP_BROWSER = "open_inapp_browser"
    OPEN_SYSTEM_BROWSER = "open_system_browser"
    OPEN_EXTERNAL_APP = "open_external_app"
    SUBMIT_ACTION = "submit_action"
    CALL_MODAL = "call_modal"


@unique
class HeaderStyle(StrEnum):
    BLUE = 'blue'
    RED = 'red'
    YELLOW = 'yellow'


@unique
class BlockKitType(StrEnum):
    MESSAGE = 'message'
    MODAL = 'modal'


class SelectBlockOption(NamedTuple):
    text: str
    value: str

    def to_dict(self) -> Dict[str, Any]:
        return self._asdict()

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'SelectBlockOption':
        if not value:
            raise NoValueError('No value to type cast')
        return cls(**value)


class Block(ABC):
    def __init__(self, *, type: BlockType):
        self.type = type

    def __str__(self):
        return self.to_json()

    def __repr__(self):
        return str(self)

    def __eq__(self, value: object) -> bool:
        if not isinstance(value, Block):
            return False
        if self.type != value.type:
            return False
        if self.to_dict() != value.to_dict():
            return False
        return True

    def to_dict(self) -> Dict[str, Any]:
        return {'type': self.type}

    def to_json(self) -> str:
        return json.dumps(self.to_dict(), default=json_default)

    @abstractmethod
    def validate(self) -> bool:
        raise NotImplementedError()

    @classmethod
    @abstractclassmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'Block':
        raise NotImplementedError()


class TextBlock(Block):
    max_len_text = 500

    def __init__(self, *, text: str, markdown: Optional[bool] = False):
        super().__init__(type=BlockType.TEXT)
        self.text = text
        self.markdown = markdown if markdown is not None else False

    def to_dict(self):
        return dict(
            super().to_dict(),
            text=self.text,
            markdown=self.markdown,
        )

    def validate(self) -> bool:
        if not self.text or len(self.text) > self.max_len_text:
            return False
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'TextBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' not in value or value['type'] != BlockType.TEXT:
            raise InvalidBlockType('No type or invalid')
        value = {k: v for k, v in value.items() if k != 'type'}

        return cls(**dict(
            value,
            markdown=value['markdown'] if exist_kv('markdown', value) else False,
        ))


class ImageLinkBlock(Block):
    def __init__(self, *, url: str):
        super().__init__(type=BlockType.IMAGE_LINK)
        self.url = url

    def to_dict(self):
        return dict(
            super().to_dict(),
            url=self.url,
        )

    def validate(self) -> bool:
        if not self.url:
            return False
        o = urlparse(self.url)
        if not (o.scheme and o.netloc and o.path):
            return False
        if not os.path.splitext(o.path)[1]:
            return False
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'ImageLinkBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' not in value or value['type'] != BlockType.IMAGE_LINK:
            raise InvalidBlockType('No type or invalid')
        value = {k: v for k, v in value.items() if k != 'type'}

        return cls(**value)


class ButtonBlock(Block):
    max_len_text = 20

    def __init__(self,
                 *,
                 text: str,
                 style: ButtonStyle,
                 action_type: Optional[ButtonActionType] = None,
                 action_name: Optional[str] = None,
                 value: Optional[str] = None):
        super().__init__(type=BlockType.BUTTON)
        self.text = text
        self.style = style
        self.action_type = action_type
        self.action_name = action_name
        self.value = value

    def to_dict(self):
        kwargs = dict(
            super().to_dict(),
            text=self.text,
            style=self.style,
            action_type=self.action_type,
            action_name=self.action_name,
            value=self.value,
        )
        return {k: v for k, v in kwargs.items() if v is not None}

    def validate(self) -> bool:
        if not self.text or len(self.text) > self.max_len_text:
            return False
        if not self.style:
            return False
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'ButtonBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' not in value or value['type'] != BlockType.BUTTON:
            raise InvalidBlockType('No type or invalid')
        value = {k: v for k, v in value.items() if k != 'type'}

        return cls(**dict(
            value,
            style=ButtonStyle(value['style']),
            action_type=ButtonActionType(value['action_type']) if exist_kv('action_type', value) else None,
        ))


class DividerBlock(Block):
    def __init__(self) -> None:
        super().__init__(type=BlockType.DIVIDER)

    def to_dict(self):
        return super().to_dict()

    def validate(self) -> bool:
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'DividerBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' in value and value['type'] != BlockType.DIVIDER:
            raise InvalidBlockType('No type or invalid')

        return cls()


class HeaderBlock(Block):
    max_len_text = 20

    def __init__(self, *, text: str, style: Optional[HeaderStyle] = None):
        super().__init__(type=BlockType.HEADER)
        self.text = text
        self.style = style

    def to_dict(self):
        return dict(
            super().to_dict(),
            text=self.text,
            style=self.style,
        )

    def validate(self) -> bool:
        if not self.text or len(self.text) > self.max_len_text:
            return False
        if not self.style:
            return False
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'HeaderBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' not in value or value['type'] != BlockType.HEADER:
            raise InvalidBlockType('No type or invalid')
        value = {k: v for k, v in value.items() if k != 'type'}

        return cls(**dict(
            value,
            style=HeaderStyle(value['style']) if exist_kv('style', value) else None,
        ))


class ActionBlock(Block):
    max_len_elements = 3

    def __init__(self, *, elements: List[ButtonBlock]):
        super().__init__(type=BlockType.ACTION)
        self.elements = elements or []

    def to_dict(self):
        return dict(
            super().to_dict(),
            elements=[item.to_dict() for item in self.elements],
        )

    def validate(self) -> bool:
        if not self.elements or len(self.elements) > self.max_len_elements:
            return False
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'ActionBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' not in value or value['type'] != BlockType.ACTION:
            raise InvalidBlockType('No type or invalid')
        value = {k: v for k, v in value.items() if k != 'type'}

        return cls(**dict(
            value,
            elements=[ButtonBlock.from_dict(item) for item in value['elements']],
        ))


class DescriptionBlock(Block):
    max_len_term = 10

    def __init__(self, *, term: str, content: TextBlock, accent: Optional[bool] = False):
        super().__init__(type=BlockType.DESCRIPTION)
        self.term = term
        self.content = content
        self.accent = accent if accent is not None else False

    def to_dict(self):
        return dict(
            super().to_dict(),
            term=self.term,
            content=self.content.to_dict(),
            accent=self.accent,
        )

    def validate(self) -> bool:
        if not self.term or len(self.term) > self.max_len_term:
            return False
        if not self.content:
            return False
        if self.accent is None:
            return False
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'DescriptionBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' not in value or value['type'] != BlockType.DESCRIPTION:
            raise InvalidBlockType('No type or invalid')
        value = {k: v for k, v in value.items() if k != 'type'}

        return cls(**dict(
            value,
            content=TextBlock.from_dict(value['content']),
            accent=value['accent'] if exist_kv('accent', value) else False,
        ))


class SectionBlock(Block):
    def __init__(self, *, content: TextBlock, accessory: ImageLinkBlock):
        super().__init__(type=BlockType.SECTION)
        self.content = content
        self.accessory = accessory

    def to_dict(self):
        return dict(
            super().to_dict(),
            content=self.content.to_dict(),
            accessory=self.accessory.to_dict(),
        )

    def validate(self) -> bool:
        if not self.content:
            return False
        if not self.accessory:
            return False
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'SectionBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' not in value or value['type'] != BlockType.SECTION:
            raise InvalidBlockType('No type or invalid')
        value = {k: v for k, v in value.items() if k != 'type'}

        return cls(**dict(
            value,
            content=TextBlock.from_dict(value['content']),
            accessory=ImageLinkBlock.from_dict(value['accessory']),
        ))


class ContextBlock(Block):
    def __init__(self, *, content: TextBlock, image: ImageLinkBlock):
        super().__init__(type=BlockType.CONTEXT)
        self.content = content
        self.image = image

    def to_dict(self):
        return dict(
            super().to_dict(),
            content=self.content.to_dict(),
            image=self.image.to_dict(),
        )

    def validate(self) -> bool:
        if not self.content:
            return False
        if not self.image:
            return False
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'ContextBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' not in value or value['type'] != BlockType.CONTEXT:
            raise InvalidBlockType('No type or invalid')
        value = {k: v for k, v in value.items() if k != 'type'}

        return cls(**dict(
            value,
            content=TextBlock.from_dict(value['content']),
            image=ImageLinkBlock.from_dict(value['image']),
        ))


class LabelBlock(Block):
    max_len_text = 200

    def __init__(self, *, text: str, markdown: bool):
        super().__init__(type=BlockType.LABEL)
        self.text = text
        self.markdown = markdown

    def to_dict(self):
        return dict(
            super().to_dict(),
            text=self.text,
            markdown=self.markdown,
        )

    def validate(self) -> bool:
        if not self.text or len(self.text) > self.max_len_text:
            return False
        if self.markdown is None:
            return False
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'LabelBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' not in value or value['type'] != BlockType.LABEL:
            raise InvalidBlockType('No type or invalid')
        value = {k: v for k, v in value.items() if k != 'type'}

        return cls(**value)


class InputBlock(Block):
    max_len_placeholder = 50

    def __init__(self, *, name: str, required: Optional[bool] = False, placeholder: Optional[str] = None):
        super().__init__(type=BlockType.INPUT)
        self.name = name
        self.required = required if required is not None else False
        self.placeholder = placeholder

    def to_dict(self):
        return dict(
            super().to_dict(),
            name=self.name,
            required=self.required,
            placeholder=self.placeholder,
        )

    def validate(self) -> bool:
        if not self.name:
            return False
        if self.placeholder and len(self.placeholder) > self.max_len_placeholder:
            return False
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'InputBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' not in value or value['type'] != BlockType.INPUT:
            raise InvalidBlockType('No type or invalid')
        value = {k: v for k, v in value.items() if k != 'type'}

        return cls(**dict(
            value,
            required=value['required'] if exist_kv('required', value) else False,
        ))


class SelectBlock(Block):
    max_len_options = 30
    max_len_placeholder = 50

    def __init__(self, *, name: str, options: List[SelectBlockOption], required: Optional[bool] = False, placeholder: Optional[str] = None):
        super().__init__(type=BlockType.SELECT)
        self.name = name
        self.options = options or []
        self.required = required if required is not None else False
        self.placeholder = placeholder

    def to_dict(self):
        return dict(
            super().to_dict(),
            name=self.name,
            options=[item.to_dict() for item in self.options],
            required=self.required,
            placeholder=self.placeholder,
        )

    def validate(self) -> bool:
        if not self.name:
            return False
        if not self.options or len(self.options) > self.max_len_options:
            return False
        if self.placeholder and len(self.placeholder) > self.max_len_placeholder:
            return False
        return True

    @classmethod
    def from_dict(cls, value: Dict[str, Any]) -> 'SelectBlock':
        if not value:
            raise NoValueError('No value to type cast')
        if 'type' not in value or value['type'] != BlockType.SELECT:
            raise InvalidBlockType('No type or invalid')
        value = {k: v for k, v in value.items() if k != 'type'}

        return cls(**dict(
            value,
            options=[SelectBlockOption.from_dict(item) for item in value['options']],
            required=value['required'] if exist_kv('required', value) else False,
        ))


class BlockKitBuilder:
    def __init__(self, *, type: BlockKitType):
        self.type = type
        self.reset()

    def reset(self):
        self.vars: Dict[str, Any] = {
            'blocks': [],
        }

    @property
    def text(self) -> str:
        if self.type != BlockKitType.MESSAGE:
            raise InvalidBlockType("It can be set only for message type")
        return self.vars['text'] if 'text' in self.vars else ''

    @text.setter
    def text(self, value: str):
        if self.type != BlockKitType.MESSAGE:
            raise InvalidBlockType("It can be set only for message type")
        self.vars['text'] = value

    @property
    def title(self) -> str:
        if self.type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        return self.vars['title'] if 'title' in self.vars else ''

    @title.setter
    def title(self, value: str):
        if self.type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        self.vars['title'] = value

    @property
    def accept(self) -> str:
        if self.type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        return self.vars['accept'] if 'accept' in self.vars else ''

    @accept.setter
    def accept(self, value: str):
        if self.type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        self.vars['accept'] = value

    @property
    def decline(self) -> str:
        if self.type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        return self.vars['decline'] if 'decline' in self.vars else ''

    @decline.setter
    def decline(self, value: str):
        if self.type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        self.vars['decline'] = value

    @property
    def value(self) -> str:
        if self.type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        return self.vars['value'] if 'value' in self.vars else ''

    @value.setter
    def value(self, value: str):
        if self.type != BlockKitType.MODAL:
            raise InvalidBlockType("It can be set only for modal type")
        self.vars['value'] = value

    @property
    def blocks(self) -> List[Block]:
        return self.vars['blocks'] if 'blocks' in self.vars and self.vars['blocks'] else []

    @blocks.setter
    def blocks(self, value: List[Block]):
        self.vars['blocks'] = []
        for item in value:
            self.add_block(item)

    def add_block(self, block: Block):
        if not block.validate():
            raise InvalidBlock()
        self.vars['blocks'].append(block)

    def to_dict(self) -> Dict[str, Any]:
        return {key: value for key, value in self.vars.items() if value is not None}

    def to_json(self) -> str:
        return json.dumps(self.to_dict(), default=json_default)



import os
import json
from datetime import datetime
from typing import Optional, Tuple, Any

import click

from kakaowork.consts import StrEnum
from kakaowork.client import Kakaowork
from kakaowork.models import (
    BaseResponse,
    UserListResponse,
    ConversationListResponse,
    DepartmentListResponse,
)
from kakaowork.blockkit import Block, BlockType
from kakaowork.utils import normalize_token, command_aliases, parse_kv_pairs


class AliasedGroup(click.Group):
    def get_command(self, ctx: click.Context, cmd_name: str):
        # Return a command immediately if exact match
        command = super().get_command(ctx, cmd_name)
        if command is not None:
            return command

        # Returns a command if it matches an alias
        aliases = command_aliases()
        if cmd_name in aliases:
            actual_cmd_name = aliases[cmd_name]
            return super().get_command(ctx, actual_cmd_name)

        ctx.fail(f"No such command: {cmd_name}.")


class BlockKitParamType(click.ParamType):
    name = 'blockkit'

    def convert(self, value: str, param: Optional[click.Parameter], ctx: Optional[click.Context]) -> Any:
        if value.startswith('@') and value.endswith('.json'):  # Load from a JSON file.
            with open(value[1:], 'r') as f:
                data = json.load(f)
        elif value[0] in ['[', '{']:  # Load form JSON string.
            data = json.loads(value)
        else:  # Parse key-value pairs with space separators
            data = parse_kv_pairs(value)
        block_cls = BlockType.block_cls(data['type'])
        return block_cls.from_dict(data)

    def __repr__(self) -> str:
        return 'BLOCKKIT'


class CLIOptions:
    def __init__(self) -> None:
        self.app_key: str = ''


def _echo(ctx: click.Context, resp: BaseResponse) -> None:
    click.echo(click.style(resp.to_plain(), fg=None if resp.success else 'red'))
    ctx.exit(0 if resp.success else 1)


BLOCKKIT = BlockKitParamType()


@click.group(name='kakaowork', cls=AliasedGroup, context_settings=dict(token_normalize_func=normalize_token))
@click.pass_context
@click.option('-k', '--app-key', default=os.environ.get('KAKAOWORK_APP_KEY'))
def cli(ctx: click.Context, app_key: str):
    if not app_key:
        click.echo(click.style('No app key! Please pass your app key using option(-k, --app-key) or environment variable($KAKAOWORK_APP_KEY)', fg='red'))
        ctx.exit(1)
    opts: CLIOptions = ctx.ensure_object(CLIOptions)
    opts.app_key = app_key


@cli.group()
@click.pass_context
def users(ctx: click.Context):
    ctx.ensure_object(CLIOptions)


@users.command(name='list', help="Show a list of details of users belonging to the workspace")
@click.pass_context
@click.option('-l', '--limit', type=int, default=10)
def users_list(ctx: click.Context, limit: int):
    opts: CLIOptions = ctx.obj
    client = Kakaowork(app_key=opts.app_key)
    r = client.users.list(limit=limit)
    if not r.success:
        return _echo(ctx, r)

    def _generate_output(resp: UserListResponse):
        yield resp.to_plain()
        while resp.cursor is not None:
            resp = client.users.list(cursor=resp.cursor)
            yield resp.to_plain()

    click.echo_via_pager(_generate_output(r))


@users.command(name='find', help="Show details of a user by identifier (ID, Email, Mobile)")
@click.pass_context
@click.option('-u', '--user-id', type=int)
@click.option('-e', '--email')
@click.option('-m', '--mobile')
def users_find(ctx: click.Context, user_id: Optional[int] = None, email: Optional[str] = None, mobile: Optional[str] = None):
    opts: CLIOptions = ctx.obj
    client = Kakaowork(app_key=opts.app_key)

    if user_id is not None:
        r = client.users.info(user_id)
    elif email:
        r = client.users.find_by_email(email)
    elif mobile:
        r = client.users.find_by_phone_number(mobile)
    else:
        raise click.BadParameter("You must enter either '--user-id (-u)' or '--email (-e)' or '--mobile (-m)")
    _echo(ctx, r)


@users.command(name='set', help="Change the user's status (working time, vacation time)")
@click.pass_context
@click.argument('user_id', type=int)
@click.option('-wt', '--work-time', nargs=2, type=click.DateTime())
@click.option('-vt', '--vacation-time', nargs=2, type=click.DateTime())
def users_set(ctx: click.Context,
              user_id: int,
              work_time: Optional[Tuple[datetime, datetime]] = None,
              vacation_time: Optional[Tuple[datetime, datetime]] = None):
    if not (work_time or vacation_time):
        raise click.BadParameter("You must enter either '--work-time (-wt)' or '--vacation-time (-vt)'")

    opts: CLIOptions = ctx.obj
    client = Kakaowork(app_key=opts.app_key)

    if work_time:
        start_time, end_time = work_time[0], work_time[1]
        r = client.users.set_work_time(user_id=user_id, work_start_time=start_time, work_end_time=end_time)
        click.echo(click.style('Set work time: ' + r.to_plain(), fg=None if r.success else 'red'))
        if not r.success:
            ctx.exit(1)

    if vacation_time:
        start_time, end_time = vacation_time[0], vacation_time[1]
        r = client.users.set_vacation_time(user_id=user_id, vacation_start_time=start_time, vacation_end_time=end_time)
        click.echo(click.style('Set vacation time: ' + r.to_plain(), fg=None if r.success else 'red'))
        if not r.success:
            ctx.exit(1)


@cli.group()
@click.pass_context
def conversations(ctx: click.Context):
    ctx.ensure_object(CLIOptions)


@conversations.command(name='open')
@click.pass_context
@click.argument('user_ids', nargs=-1, type=int)
def conversations_open(ctx: click.Context, user_ids: Tuple[int, ...]):
    opts: CLIOptions = ctx.obj
    r = Kakaowork(app_key=opts.app_key).conversations.open(user_ids=list(user_ids))
    _echo(ctx, r)


@conversations.command(name='list')
@click.pass_context
@click.option('-l', '--limit', type=int, default=10)
def conversations_list(ctx: click.Context, limit: int):
    opts: CLIOptions = ctx.obj
    client = Kakaowork(app_key=opts.app_key)
    r = client.conversations.list(limit=limit)
    if not r.success:
        return _echo(ctx, r)

    def _generate_output(resp: ConversationListResponse):
        yield resp.to_plain()
        while resp.cursor is not None:
            resp = client.conversations.list(cursor=resp.cursor)
            yield resp.to_plain()

    click.echo_via_pager(_generate_output(r))


@conversations.command(name='users')
@click.pass_context
@click.argument('conversation_id', type=int)
def conversations_users(ctx: click.Context, conversation_id: int):
    opts: CLIOptions = ctx.obj
    client = Kakaowork(app_key=opts.app_key)
    r = client.conversations.users(conversation_id=conversation_id)
    _echo(ctx, r)


@conversations.command(name='invite')
@click.pass_context
@click.argument('conversation_id', type=int)
@click.argument('user_ids', nargs=-1, type=int)
def conversations_invite(ctx: click.Context, conversation_id: int, user_ids: Tuple[int, ...]):
    opts: CLIOptions = ctx.obj
    client = Kakaowork(app_key=opts.app_key)
    r = client.conversations.invite(conversation_id=conversation_id, user_ids=list(user_ids))
    _echo(ctx, r)


@conversations.command(name='kick')
@click.pass_context
@click.argument('conversation_id', type=int)
@click.argument('user_ids', nargs=-1, type=int)
def conversations_kick(ctx: click.Context, conversation_id: int, user_ids: Tuple[int, ...]):
    opts: CLIOptions = ctx.obj
    client = Kakaowork(app_key=opts.app_key)
    r = client.conversations.kick(conversation_id=conversation_id, user_ids=list(user_ids))
    _echo(ctx, r)


@cli.group()
@click.pass_context
def messages(ctx: click.Context):
    ctx.ensure_object(CLIOptions)


@messages.command(name='send')
@click.pass_context
@click.argument('conversation_id', type=int)
@click.argument('text', type=str)
@click.option('-b', '--block', 'blocks', type=BLOCKKIT, multiple=True)
def messages_send(ctx: click.Context, conversation_id: int, text: str, blocks: Tuple[Block, ...]):
    opts: CLIOptions = ctx.obj
    r = Kakaowork(app_key=opts.app_key).messages.send(conversation_id=conversation_id, text=text, blocks=list(blocks))
    _echo(ctx, r)


@cli.group()
@click.pass_context
def departments(ctx: click.Context):
    ctx.ensure_object(CLIOptions)


@departments.command(name='list')
@click.pass_context
@click.option('-l', '--limit', type=int, default=10)
def departments_list(ctx: click.Context, limit: int):
    opts: CLIOptions = ctx.obj
    client = Kakaowork(app_key=opts.app_key)
    r = client.departments.list(limit=limit)
    if not r.success:
        return _echo(ctx, r)

    def _generate_output(resp: DepartmentListResponse):
        yield resp.to_plain()
        while resp.cursor is not None:
            resp = client.departments.list(cursor=resp.cursor)
            yield resp.to_plain()

    click.echo_via_pager(_generate_output(r))


@cli.group()
@click.pass_context
def spaces(ctx: click.Context):
    ctx.ensure_object(CLIOptions)


@spaces.command(name='info')
@click.pass_context
def spaces_info(ctx: click.Context):
    opts: CLIOptions = ctx.obj
    r = Kakaowork(app_key=opts.app_key).spaces.info()
    _echo(ctx, r)


@cli.group()
@click.pass_context
def bots(ctx: click.Context):
    ctx.ensure_object(CLIOptions)


@bots.command(name='info')
@click.pass_context
def bots_info(ctx: click.Context):
    opts: CLIOptions = ctx.obj
    r = Kakaowork(app_key=opts.app_key).bots.info()
    _echo(ctx, r)



class KakaoworkError(Exception):
    """
    Kakaowork base error
    """


class InvalidBlockType(KakaoworkError):
    """
    Invalid block type
    """


class InvalidBlock(KakaoworkError):
    """
    Invalid block
    """


class NoValueError(KakaoworkError):
    """
    No value error
    """
